; ----------------------------------------------------------------------------
; -      TITULO  : Tetris COM-MASM                                           -
; -----                                                                  -----
; -      AUTOR   : Alfonso Vctor Caballero Hurtado                          -
; -----                                                                  -----
; -      VERSION : 1.0                                                       -
; -----                                                                  -----
; -      (c) 2010. Abre los Ojos al Ensamblador                              -
; ----------------------------------------------------------------------------

__LongMax  EQU   6                ; Nmero mximo de dgitos entero
DifRetardo EQU   2                ; Diferencia en el retardo entre niveles
LinXnivel  EQU  10                ; Nmero de lneas por nivel
AnchFig    EQU  4                 ; Ancho de cada figura
AltFig     EQU  4                 ; Alto de cada figura
NumFiguras EQU  8                 ; Nmero de figuras
NumRota    EQU  4                 ; Mximo de rotaciones
Filas      EQU 14                 ; Filas del tablero
Columnas   EQU 12                 ; Columnas del tablero
ColMens    EQU 30                 ; Columna donde se escriben los mensajes (tetis, etc)
ValIni     EQU  0                 ; Valor para inicializar
ColFicha   EQU  7                 ; Color de la ficha
ColBorde   EQU  9                 ; Color del borde
CarFicha   EQU 'X'                ; Carcter que construye la ficha
FilaIni    EQU (25-Filas)/2       ; Para pintar el tablero
ColmIni    EQU 1;(80 - Columnas) / 2 - 1
ColmnIni   EQU (Columnas-AnchFig) / 2 ; Posicin relativa por donde empieza a caer la figura
CarBorde   EQU 179                ; "|"
CarBorde2  EQU 196                ; "-"
Esquina1   EQU 192                ; "|-"
Esquina2   EQU 217                ; "-|"
DescEsp    EQU 5                  ; Descenso en la espera
FilMarc    EQU 8                  ; Fila donde se pinta el marcador
FilNiv     EQU 12                 ; Fila donde se pinta el nivel
FilNext    EQU 16                 ; Fila donde se pinta la siguiente figura
ColNext    EQU 16                 ; Columna donde se pinta la siguiente figura
cVelocidad EQU 18*1               ; Constante de segundos de espera
Bloque     EQU 219
ParaAbajo  EQU 1
ParaIzqda  EQU 2
ParaRotar  EQU 3
ParaDcha   EQU 4

FigReg   STRUC
  PosX    DW  ?                   ; X-posicin
  PosY    DW  ?                   ; Y-posicin
  Rota    DW  ?                   ; Nmero de rotacin: 0, 1, 2, 3
  PosXtmp DW  ?                   ; X-posicin solicitada
  PosYtmp DW  ?                   ; Y-posicin solicitada
  Rotatmp DW  ?                   ; Rotacin solicitada
  Tipo    DW  ?                   ; Tipo de figura: 0..NumFiguras
FigReg   ENDS

; PosX indica la X-posicin de la figura
; PosY indica la Y-posicin de la figura
; Tipo indica el tipo de figura (hay NumFiguras). Con este valor, podemos
;   saber la direccin de su 0-rotacin accediendo a [PosFigs+Tipo], por
;   eso lo hemos definido como word
; Rota indica el nmero de rotacin que lleva. Con el valor [PosFigs+Tipo]
;   accedemos a la direccin de su 0-rotacin, por ejemplo "L1", entonces
;   en [L1+Rota] tenemos la direccin buscada

codigo SEGMENT PARA PUBLIC 'CODE'
  ASSUME CS:codigo, DS:codigo, ES:codigo, SS:codigo
  ORG 100h
  Entrada PROC
;     CALL      RandomizeTimer
    CALL      SaveOldInt1Ch
    CALL      SetNewInt1Ch
    CALL      Estructura
    CALL      CursOff
    $_Nueva_Partida:
    CALL      Inicializa
    $_Nueva_Figura:
    CALL      PintaSgteFigura
    CALL      NuevaFigura
    CALL      ColocaFigura
    CALL      TableroTmp2Scrn
    $_Principal:
      CALL      GetKey
      CMP       BYTE PTR [Movimiento], -2
      JZ        $_Salimos
      CMP       BYTE PTR [Movimiento], -1
      JZ        $_Nueva_Partida
      CMP       WORD PTR [Tiempo], 0
      JA        $_Sgte1
        ; Entramos aqu porque se agot el contador de tiempo: el sistema mueve abajo
        CALL      GuardaCC
        MOV       BYTE PTR [Movimiento], ParaAbajo
        INC       WORD PTR [miFigura.PosYtmp]
        MOV       AX, Velocidad
        MOV       WORD PTR [Tiempo], AX       ; Reiniciamos el contador de tiempo
      $_Sgte1:
      CMP       BYTE PTR [Movimiento], 0
      JZ        $_Sgte2
        CALL      EsPosibleMvto               ; Comprueba el movimiento
        ; ZF = 1 movimiento correcto, ZF = 0 movimiento incorrecto
        ; CF = 1 ficha con vida,      CF = 0 ficha alcanz su final
        JNC       $_Sgte3
        JNZ       $_Sgte4
        CALL      NuevasCC          ; Hacemos realidad lo solicitado
        CALL      MueveFicha        ; Y ponemos la ficha segn nuevas CC
      $_Sgte2:
    JMP       $_Principal
    $_Sgte3:
      .IF         WORD PTR [miFigura.PosY] == 0
        XOR         AH, AH
        INT         16h
        CMP         AH, 1    ; ESC - Exit
        JNZ         $_Nueva_Partida
        JMP         $_Salimos
      .ENDIF
      MOV         BYTE PTR [Movimiento], 0
      CALL        CopiaTmpATablero
      CALL        RastreaFilas       ; Hay filas completas que eliminar?
      JMP         $_Nueva_Figura
    $_Sgte4:
      MOV       BYTE PTR [Movimiento], 0
      JMP       $_Principal
    $_Salimos:
    CALL      CursOn
    CALL      RestoreOldInt1Ch
    MOV       AX, 4C00h
    INT       21h
  Entrada ENDP

  SaveOldInt1Ch PROC NEAR
    ; Propsito: Guarda la direccin de la antigua ISR de Int 1Ch
    ; entrada  : OldInt1Ch
    ; salida   : Ninguna
    ; Destruye : AX
    MOV       AX, 351Ch                   ; Obtiene la direccin de la
    INT       21h                         ;     interrupcin 1Ch
    MOV       WORD PTR OldInt1Ch[2],ES    ; Guarda segmento
    MOV       WORD PTR OldInt1Ch[0],BX    ;     y direccin
    RET
  SaveOldInt1Ch ENDP

  RestoreOldInt1Ch PROC NEAR
    ; Propsito: Restaura la direccin de la antigua ISR de Int 1Ch
    ; entrada  : OldInt1Ch
    ; salida   : Ninguna
    ; Destruye : AX, DX
    PUSH      DS                          ; Guardamos DS
    LDS       DX, OldInt1Ch               ; Carga la direccin original
    MOV       AX, 251Ch                   ; Lo restaura a la tabla de vectores
    INT       21h
    POP       DS
    RET
  RestoreOldInt1Ch ENDP

  SetNewInt1Ch PROC NEAR
    ; Propsito: Establece la direccin de la nueva ISR para Int 1Ch
    ; entrada  : NuevaInt1Ch
    ; salida   : Ninguna
    ; Destruye : AX, DX
    MOV       DX, OFFSET NuevaInt1Ch      ; Carga la direccin de nueva rutina
    MOV       AX, 251Ch                   ; Establece la nueva interrupcin
    INT       21h
    RET
  SetNewInt1Ch ENDP

  NuevaInt1Ch PROC FAR
    ; Propsito: Nueva ISR para la INT 1Ch, crea un retardo
    ; entrada  : KeyTable
    ; salida   : KeyTable
    ; Destruye : Ninguna
    PUSHF
    CMP       WORD PTR [CS:Tiempo], 0
    JZ        $_Fin
      DEC       WORD PTR [CS:Tiempo]      ; Decrementamos el contador de pulsos
    $_Fin:
    POPF
    JMP       [CS:OldInt1Ch]              ; Saltamos a la vieja rutina
  NuevaInt1Ch  ENDP

  CursOff         PROC
    ; Propsito: Oculta el cursor
    ; entrada  : TamCursor
    ; salida   : ninguna
    ; Destruye : ninguna
    PUSH      AX
    PUSH      BX
    PUSH      CX
    MOV       AH, 03h                       ; Obtiene posicin y tamao
    MOV       BH, 00h                       ; Pgina 0
    INT       10h
    MOV       WORD PTR [TamCursor], CX
    MOV       AH, 01                        ; Establece tamao del cursor
    OR        CH, 00100000b                 ; Bits 6, 5:
    AND       CH, 10111111b                 ;     01 cursor invisible
    INT       10h                           ;     00        normal
    POP       CX
    POP       BX
    POP       AX
    RET
  CursOff         ENDP
  
  CursOn          PROC
    ; Propsito: Restablece el cursor con su tamao original
    ; entrada  : TamCursor
    ; salida   : ninguna
    ; Destruye : ninguna
    PUSH      AX
    PUSH      CX
    MOV       AH, 1                         ; Funcin selecciona tamao del cursor
    MOV       CX, WORD PTR [TamCursor]      ; Recupera el tamao original
    INT       10h
    POP       CX
    POP       AX
    RET
  CursOn          ENDP

  DesgranaWord PROC
    ; Propsito: Convierte un nmero WORD en ASCII y lo mete en variable local
    ; Entrada  : AX recoge el nmero, BP puntero al bfer local
    ; Salida   : BP apunta al bfer local donde guardamos los dgitos en ASCII
    ; Destruye : AX, BX, DX, BP
    MOV       BX, 0Ah                ; Dividir por 10 para extraer los dgitos
    MOV       BYTE PTR SS:[BP], "$"  ; Acaba en "$" para imprimirlo con INT21h
    DEC       BP
    $_Vuelta:
      XOR       DX, DX               ; DX debe ser nulo para dividir AX por BX
      DIV       BX                   ; DX:AX es resto:cociente
      OR        DX, 30h              ; Convertimos el dgito en ASCII
      MOV       BYTE PTR SS:[BP], DL ; Guardamos en la pila el resto de la divisin
      DEC       BP                   ; Apuntamos a la siguiente posicin
      OR        AX, AX               ; Comprobamos que no quedan ms dgitos
    JNZ       $_Vuelta
    INC       BP
    RET
  DesgranaWord ENDP

  WriteWord PROC
    ; Propsito: Convierte Numero en ASCIIZ$ en variable local y lo imprime
    ; Entrada  : AX: nmero word positivo
    ; Salida   : Impresin del nmero como ASCII
    ; Destruye : Ninguna
    PUSH      BP                         ; Salvamos el puntero base
    MOV       BP, SP                     ; Cargamos la pila en el puntero base
    SUB       SP, __LongMax              ; Reservamos espacio para variable local
    PUSH      AX
    PUSH      BX
    PUSH      DX
    PUSH      DS
    CALL      DesgranaWord               ; Convertimos el word a ASCII en vble local
    ; Imprimimos el word guardado en la variable local
    MOV       AX, SS
    MOV       DS, AX
    MOV       AH, 9
    MOV       DX, BP
    INT       21h
    POP       DS
    POP       DX
    POP       BX
    POP       AX
    ADD       SP, __LongMax              ; Limpia vbles locales
    POP       BP                         ; Restaura registros
    RET
  WriteWord  ENDP

  Locate PROC
    ; Propsito: Posiciona el cursor en la fila DH y columna DL
    ; Entrada  : DX: Posicin del cursor; DH fila, DL columna
    ; Salida   : ninguna
    ; Destruye : ninguna
    PUSH    AX
    PUSH    BX
    PUSH    DX
    MOV     AH, 2                         ; Funcin posicionar
    XOR     BH, BH                        ; Siempre pgina 0
    INT     10h
    POP     DX
    POP     BX
    POP     AX
    RET
  Locate ENDP

  WriteChar PROC
    ; Propsito : Pinta por pantalla una matriz de "#" segn caracter
    ; Entrada   : En pila: Posicin, Caracter a dibujar
    ; Salida    : Ninguna
    ; Destruye  : Ninguna
    PUSH      BP
    MOV       BP, SP                      ; BP apunta a los parmetros entrada
    PUSH      AX                          ; Guardamos registros
    PUSH      BX
    PUSH      CX
    PUSH      DX
    PUSH      DI
    PUSH      ES
    MOV       AX, DEFSEG                  ; Accedemos al segmento
    MOV       ES, AX
    MOV       CL, 0                       ; Inicializamos contador
    ; CHAR
    $_BByte:
      ; BL = OFFSET + CL + Caracter * 8
      MOV       AX, WORD PTR [BP+4]       ; Caracter a dibujar
      SHL       AX, 1
      SHL       AX, 1
      SHL       AX, 1
      ADD       AL, CL
      ADC       AH, 0
      MOV       DI, DEFOFF
      ADD       DI, AX
      MOV       BL, BYTE PTR ES:[DI]
      MOV       DX, WORD PTR [BP+6]       ; Posicin donde dibujar
      ADD       DH, CL                    ; Fila
      CALL      Locate                    ; Posicionamos cursor
      MOV       CH, 0                     ; Inicializamos contador
      $_BBite:
        MOV       DL, " "                 ; Suponemos que no hay nada que pintar
        SHL       BL, 1                   ; Sacamos bit al CF
        JNC       $_Siguiente             ;    estaba activo?
          MOV       DL, Bloque            ;    s
        $_Siguiente:                      ;    no
        MOV       AH, 2                   ; Escribe carcter
        INT       21h
        INC       CH                      ; Incrementamos contador
        CMP       CH, 8                   ; Hemos alcanzado los 8 bits?
      JNZ       $_BBite
      INC       CL                        ; Incrementamos contador
      CMP       CL, 8                     ; Hemos alcanzado los 8 bits?
    JNZ       $_BByte
    POP       ES                          ; Recuperamos registros
    POP       DI
    POP       DX
    POP       CX
    POP       BX
    POP       AX
    POP       BP
    RET       4
  WriteChar ENDP
  
  WriteText PROC
    ; Propsito : Pinta en pantalla una cadena de texto
    ; Entrada   : SI: Direccin del texto a pintar
    ; Salida    : Ninguna
    ; Destruye  : Ninguna
    PUSH      AX
    PUSH      BX
    $_Bucle:
      LODSB                               ; AL = [ES:DI]
      CMP       AL, 0                     ; Fin del texto?
      JE        $_Fin                     ;   s, salimos
      PUSH      BX                        ; Primer parmetro = coordenadas
      ADD       BL, 8                     ; Siguiente columna
      XOR       AH, AH
      PUSH      AX                        ; Siguiente parmetro = caracter pinta
      CALL      WriteChar
    JMP       $_Bucle
    $_Fin:
    POP       AX
    POP       BX
    RET
  WriteText ENDP

  SystFont  PROC
    ; Propsito : Recoge las direcciones de los caracteres del sistema
    ; Entrada   : Ninguna
    ; Salida    : DEFSEG, DEFOFF
    ; Destruye  : Ninguna
    PUSH      BX
    PUSH      BP
    PUSH      ES
    MOV       AX, 1130h
    MOV       BH, 03h
    INT       10h
    MOV       WORD PTR [DEFSEG], ES
    MOV       WORD PTR [DEFOFF], BP
    POP       ES
    POP       BP
    POP       BX
    RET
  SystFont  ENDP

  Estructura PROC
    ; Propsito: Dibuja el tablero y todo aquello que no vara
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    ; Nos situamos en la fila/columna para pintar el tablero
    CALL      SystFont
    MOV       SI, OFFSET Texto1
    MOV       BX, ColMens                 ; Columna
    CALL      WriteText                   ; "TETRIS"
    ;
    MOV       DH, 10                      ; fila
    MOV       DL, ColMens+1               ; columna
    CALL      Locate
    MOV       AH, 9
    MOV       DX, OFFSET Texto2
    INT       21h
    ;
    MOV       DH, 11                      ; fila
    MOV       DL, ColMens+1               ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto3
    INT       21h
    ;
    MOV       DH, 13                      ; fila
    MOV       DL, ColMens+1               ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto4
    INT       21h
    ;
    MOV       DH, 15                      ; fila
    MOV       DL, ColMens+1               ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto5
    INT       21h
    ;
    MOV       DH, 16                      ; fila
    MOV       DL, ColMens+1               ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto6
    INT       21h
    ; Siguiente
    MOV       DH, FilNext-2               ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto7
    INT       21h
    ; Nivel
    MOV       DH, FilNiv-2                ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto8
    INT       21h
    ; Marcador
    MOV       DH, FilMarc-2               ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       DX, OFFSET Texto9
    INT       21h
    ;
    MOV       BH, 0                       ; Pgina de vdeo 0
    MOV       DH, FilaIni                 ; Lnea
    MOV       AH, 2                       ; Funcin posiciona el cursor y escribe char
    MOV       CX, Filas
    $_Bucle:
      MOV       DL, ColmIni               ; Columna de la izquierda
      INT       10h                       ; Posiciona el cursor
      MOV       DL, CarBorde
      INT       21h                       ; Escribe el carcter
      MOV       DL, ColmIni+Columnas+1    ; Columna de la derecha
      INT       10h                       ; Posiciona el cursor
      MOV       DL, CarBorde
      INT       21h                       ; Escribe el carcter
      INC       DH                        ; Siguiente fila
    LOOP      $_Bucle
    MOV       DL, ColmIni                 ; Columna de la izquierda
    INT       10h                         ; Posiciona el cursor
    MOV       DL, Esquina1
    INT       21h                         ; Escribe el carcter
    MOV       DL, CarBorde2
    MOV       CX, Columnas
    $_Bucle2:
      INT       21h                       ; Escribe el carcter
    LOOP      $_Bucle2
    MOV       DL, Esquina2
    INT       21h                         ; Escribe el carcter

    RET
  Estructura ENDP

  EliminaFila PROC
    ; Propsito: Rastrea la fila BX de Tablero, porque est completa
    ; entrada  : BX: apunta a la fila a eliminar, DX: filas que quedan
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      BX
    PUSH      CX
    PUSH      DX
    PUSH      SI
    PUSH      DI
    PUSH      DS
    PUSH      ES
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    ; Estamos en la fila superior?
    OR        DX, DX
    JZ        $_Fila0
      $_Bucle_F:
        MOV       DI, BX             ; DI es el destino, donde apunta BX
        MOV       SI, BX             ; SI = una fila superior a BX
        SUB       SI, Columnas
        MOV       CX, Columnas       ; Nmero de columnas de cada fila
        REP       MOVSB              ; Copiamos la fila de SI en DI
        SUB       BX, Columnas       ; Nos movemos una fila arriba
        DEC       DX                 ; Decrementamos el contador de filas
        CMP       DX, 1
      JNZ       $_Bucle_F
    $_Fila0:
      MOV       DI, BX
      MOV       CX, Columnas
      MOV       AL, 0
      REP       STOSB
    $_Fin:
    POP       ES
    POP       DS
    POP       DI
    POP       SI
    POP       DX
    POP       CX
    POP       BX
    RET
  EliminaFila ENDP

  IncMarcador PROC
    ; Propsito: Incrementa el marcador en funcin del nivel
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    MOV       AL, BYTE PTR [Nivel]
    XOR       AH, AH
    ADD       WORD PTR [Marcador], AX
    POP       AX
    RET
  IncMarcador ENDP

  IncNivel PROC
    ; Propsito: Incrementa el nivel, si le pertenece
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    .IF       BYTE PTR [ContLineas] == 0           ; Si hemos completado la fase
      INC       BYTE PTR [Nivel]                   ; Incrementamos el nivel
      SUB       WORD PTR [Velocidad], DifRetardo   ; Decrementamos la espera de tiempo
      .IF       WORD PTR [Velocidad] < 0           ; Nos aseguramos de que no sea negativo
        MOV       WORD PTR [Velocidad], 0
      .ENDIF
      MOV       BYTE PTR [ContLineas], LinXnivel   ; Reiniciamos el contador de lneas
      MOV       AX, WORD PTR [Velocidad]           ; Reiniciamos el contador de tiempo
      MOV       WORD PTR [Tiempo], AX
    .ENDIF
    POP       AX
    RET
  IncNivel ENDP

  RastreaFilas PROC
    ; Propsito: Rastrea las filas de Tablero en busca de las completas, que eliminar
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    ; BX = OFFSET Tablero + Filas + Columnas (BX apunta a la ltima fila de Tablero)
    MOV       BX, Filas-1
    MOV       AX, Columnas
    MUL       BX
    MOV       BX, AX
    ADD       BX, OFFSET Tablero
    CLD                               ; Incrementamos DI
    ; Rastreamos las filas buscando las que estn completas para eliminarlas
    MOV       DX, Filas-1
    $_Bucle_F:
      MOV       DI, BX                ; DI apunta a la fila que exploraremos
      MOV       CX, Columnas
      MOV       AL, 1                 ; CMP AL, ES:DI CX-veces mientras sean iguales
      REPE      SCASB
      JNZ       $_Next1
        CALL      EliminaFila
        CALL      IncMarcador         ; Incrementa el marcador
        .IF       BYTE PTR [ContLineas] > 0
          DEC       BYTE PTR [ContLineas]
        .ENDIF
        CALL      IncNivel            ; Incrementa el nivel, si toca
        ; Escribimos el marcador y el nivel
        CALL      EscribeNivel
        CALL      EscribeMarcador
        INC       DX                  ; De esta manera volvemos a chequear la misma fila
        ADD       BX, Columnas        ;
      $_Next1:
      SUB       BX, Columnas
      DEC       DX
    JNZ       $_Bucle_F
    CALL      CopiaTableroATmp
    POP       DS
    POP       ES
    RET
  RastreaFilas ENDP

  Inicializa PROC
    ; Propsito: Inicializa los objetos antes de cada partida
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    CLD
    ; Inicializamos Tablero
    MOV       CX, Filas*Columnas
    MOV       AL, ValIni
    MOV       DI, OFFSET Tablero
    REP       STOSB
    ; Inicializamos TableroTmp
    MOV       CX, Filas*Columnas
    MOV       AL, ValIni
    MOV       DI, OFFSET TableroTmp
    REP       STOSB
    ; Inicializamos el nivel y el marcador
    MOV       BYTE PTR [Nivel], 1
    MOV       WORD PTR [Marcador], 0
    MOV       WORD PTR [Velocidad], cVelocidad
    ; Calculamos la ficha aleatoria y la siguiente
    MOV       CL, NumFiguras
    CALL      Random
    MOV       BYTE PTR [Figura], DL
    MOV       CL, NumFiguras
    CALL      Random
    MOV       BYTE PTR [Figura+1], DL
    ; Ningn movimiento para la figura
    MOV       BYTE PTR [Movimiento], 0
    ; Inicializamos el contador de lneas por nivel
    MOV       BYTE PTR [ContLineas], LinXnivel
    ; Inicializamos el contador de tiempo
    MOV       WORD PTR [Velocidad], cVelocidad  ; Reiniciamos el contador de tiempo
    MOV       WORD PTR [Tiempo], cVelocidad     ; con el valor original
    ; Escribimos el marcador y el nivel
    CALL      EscribeNivel
    CALL      EscribeMarcador
    POP       DS
    POP       ES
    RET
  Inicializa ENDP
  
  PreparaBuf PROC
    ; Propsito: Preparamos el TableroTmp con char y color para volcar a scrn
    ;            puesto que trabajar con memoria ram es ms rpido que mem vdeo
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       SI, OFFSET TableroTmp
    MOV       DI, OFFSET TableroBuf
    MOV       CX, Filas*Columnas
    CLD
    $_Bucle:
      LODSB
      CMP       AL, ValIni
      MOV       AL, ' '
      MOV       AH, ColFicha           ; Color de la ficha
      JZ        $_Sgte
      MOV       AL, CarFicha
      $_Sgte:
      STOSW
    LOOP      $_Bucle
    POP       DS
    POP       ES
    RET
  PreparaBuf ENDP
  
  VolcarBuf PROC
    ; Propsito: Vuelca el contenido del TableroBuf en la pantalla
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       AX, 0B800h
    MOV       ES, AX
    MOV       SI, OFFSET TableroBuf
    MOV       BX, FilaIni*80*2+(ColmIni+1)*2
    MOV       DL, Filas
    CLD
    $_BFilas:
      MOV       DI, BX
      MOV       CX, Columnas
      REP       MOVSW
      ADD       BX, 80*2
      DEC       DL
    JNZ       $_BFilas
    POP       DS
    POP       ES
    RET
  VolcarBuf ENDP
  
  TableroTmp2Scrn PROC
    ; Propsito: Vuelca el contenido de tablero en la pantalla
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    CALL      PreparaBuf
    CALL      VolcarBuf
    RET
  TableroTmp2Scrn ENDP

;   RandomizeTimer PROC
;     ; Propsito: Establece una semilla aleatoria en funcin del tiempo
;     ; entrada  : CX
;     ; salida   : DL: nmero aleatorio
;     ; Destruye : AX, BX, CX, DX
;     XOR       AH, AH
;     INT       1Ah
;     MOV       WORD PTR [RandSeed], DX
;     RET
;   RandomizeTimer ENDP

  Random PROC
    ; Propsito: Calcula un nmero pseudoaleatorio entre [0..CL-1]
    ; entrada  : CX
    ; salida   : DL: nmero aleatorio
    ; Destruye : AX DX
    XOR       AX, AX
    OUT       43h, AL
    IN        AL, 40h
    XOR       AH, AH
    DIV       CL
    MOV       DL, AH
    RET
  Random ENDP

  PintaSgteFigura PROC
    ; Propsito: Pinta la figura siguiente
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       BL, BYTE PTR [Figura+1]     ; Apuntamos a la siguiente figura
    XOR       BH, BH
    SHL       BX, 1
    MOV       SI, OFFSET PosFigs          ; SI apunta a la direccin en PosFigs
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la 1 direccin en L1,L2,Z1,Z2,B1,C1
    MOV       SI, WORD PTR [SI]           ; SI apunta a la 1 definicin de la figura
    MOV       BH, 0                       ; Pgina de vdeo 0
    MOV       DH, FilNext                 ; Lnea
    MOV       AH, 2                       ; Funcin posiciona el cursor y escribe char
    MOV       CL, AltFig                  ; Nmero de filas de la figura
    $_BucleF:
      MOV       DL, ColNext               ; Columna de la izquierda
      INT       10h                       ; Posiciona el cursor
      MOV       CH, AnchFig               ; Nmero de columnas de la figura
      $_BucleC:
        MOV       DL, BYTE PTR DS:[SI]
        CMP       DL, ValIni
        JZ        $_Next1
        MOV       DL, CarFicha
        $_Next1:
        INT       21h                     ; Escribe el carcter
        INC       SI
        DEC       CH
      JNZ       $_BucleC
      INC       DH                        ; Siguiente fila
      DEC       CL
    JNZ       $_BucleF
    POP       DS
    POP       ES
    RET
  PintaSgteFigura ENDP

  EscribeMarcador PROC
    ; Propsito: Escribe el marcador
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    PUSH      DX
    MOV       DH, FilMarc                 ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       AH, 9
    MOV       DX, OFFSET Blanco
    INT       21h
    MOV       DH, FilMarc                 ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       AX, WORD PTR [Marcador]
    CALL      WriteWord
    POP       DX
    POP       AX
    RET
  EscribeMarcador ENDP

  EscribeNivel PROC
    ; Propsito: Escribe el nivel
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    PUSH      DX
    MOV       DH, FilNiv                  ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       AH, 9
    MOV       DX, OFFSET Blanco
    INT       21h
    MOV       DH, FilNiv                  ; fila
    MOV       DL, ColNext                 ; columna
    CALL      Locate
    MOV       AL, BYTE PTR [Nivel]
    XOR       AH, AH
    CALL      WriteWord
    POP       DX
    POP       AX
    RET
  EscribeNivel ENDP

  ColocaFigura PROC
    ; Propsito: Coloca miFigura dentro de TableroTmp
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       BX, WORD PTR [miFigura.Tipo]
    SHL       BX, 1
    MOV       SI, OFFSET PosFigs
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la 1 direccin en L1,L2,Z1,Z2,B1,C1
    MOV       BX, WORD PTR [miFigura.Rota]
    SHL       BX, 1
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la BX-definicin de la figura
    MOV       BX, WORD PTR [miFigura.PosY]
    MOV       AX, Columnas
    MUL       BX
    MOV       BX, AX
    ADD       BX, OFFSET TableroTmp
    ADD       BX, WORD PTR [miFigura.PosX]
    MOV       DX, AltFig
    CLD
    $_Filas:
      MOV       DI, BX
      MOV       CX, AnchFig
      $_Columnas:
        LODSB
        OR        AL, AL
        JZ        $_Sgte
          MOV       ES:[DI], AL
        $_Sgte:
        INC       DI
      LOOP      $_Columnas
      ADD       BX, Columnas
      DEC       DX
    JNZ       $_Filas
    POP       DS
    POP       ES
    RET
  ColocaFigura ENDP

  NuevaFigura PROC
    ; Propsito: Inicializamos objetos para una nueva figura
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    ; Definimos miFicha
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       AL, BYTE PTR [Figura]
    XOR       AH, AH
    MOV       WORD PTR [miFigura.Tipo], AX    ; Recogemos la figura para miFicha
    MOV       WORD PTR [miFigura.Rota], 0
    MOV       WORD PTR [miFigura.PosX], ColmnIni ;   -1
    MOV       WORD PTR [miFigura.PosY], 0  ;    Filas-2  
    ; Pasamos la siguiente figura a la primera posicin
    MOV       AL, BYTE PTR [Figura+1]
    MOV       BYTE PTR [Figura], AL
    ; Calculamos la siguiente ficha aleatoria
    MOV       CL, NumFiguras
    CALL      Random
    MOV       BYTE PTR [Figura+1], DL
    POP       DS
    POP       ES
    RET
  NuevaFigura ENDP

  Superpone PROC
    ; Propsito: Chequeamos si hay byte encendido de la figura sobre byte del tablero
    ;            Si fuera as, el movimiento no sera vlido
    ; entrada  : Ninguna
    ; salida   : ZF=1 si no hay superposicin (todo bien), ZF = 0 si la hay (mvto incorrecto)
    ; Destruye : Ninguna
    PUSH      DS
    MOV       AX, CS
    MOV       DS, AX
    ; 1) SI: Apuntamos a la definicin real de la figura
    MOV       BX, WORD PTR [miFigura.Tipo]
    SHL       BX, 1                       ; Porque tratamos con words
    MOV       SI, OFFSET PosFigs          ; Direccin de matriz de direcciones
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la 1 direccin en L1,L2,Z1,Z2,B1,C1
    MOV       BX, WORD PTR [miFigura.Rotatmp]
    SHL       BX, 1                       ; Porque tratamos con words
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la BX-definicin de la figura
    ; 2) BX: Apuntamos a la posicin del tablero donde est la figura
    MOV       BX, WORD PTR [miFigura.PosYtmp]
    MOV       AX, Columnas
    MUL       BX
    MOV       BX, AX
    ADD       BX, OFFSET Tablero
    ADD       BX, WORD PTR [miFigura.PosXtmp]
    ; 3) Chequeamos si hay bytes encendidos superpuestos
    MOV       CL, AltFig                  ; Nmero de filas de la figura
    $_BucleF:
      MOV       CH, AnchFig               ; Nmero de columnas de la figura
      MOV       DI, BX                    ; Recogemos la fila del tablero
      $_BucleC:
        MOV       AL, BYTE PTR DS:[DI]
        AND       AL, BYTE PTR DS:[SI]
        JNZ       $_Fin
        INC       DI                      ; Siguiente byte en el tablero
        INC       SI                      ; Siguiente byte en la figura
        DEC       CH                      ; Decrementamos 1 en la anchura
      JNZ       $_BucleC
      ADD       BX, Columnas              ; Siguiente fila en tablero
      DEC       CL                        ; Decrementamos 1 en la altura
    JNZ       $_BucleF
    ; Llegamos aqu cuando todo ha ido bien y ZF = 1 porque hemos salido del bucle
    STC                                   ; Todo bien
    $_Fin:
    POP       DS
    RET
  Superpone ENDP

  CheckInf PROC
    ; Propsito: Chequeamos si la figura sobrepas la parte inferior del tablero
    ; entrada  : BX: Apunta a la definicin real de la figura
    ; salida   : CarryFlag = 1 -> con vida, CarryFlag = 0 -> fin del movimiento
    ; Destruye : Ninguna
    PUSH      BX
    PUSH      DS
    PUSH      ES
    MOV       AX, CS
    MOV       DS, AX
    MOV       ES, AX
    MOV       DX, miFigura.PosYtmp
    SUB       DX, Filas - (AltFig-1)
    JL        $_Fin
    STD                                   ; Decrementamos DI
    MOV       DI, BX
    ADD       DI, AnchFig * AltFig-1      ; BX apunta a la parte inferior de la figura
    $_BucleF:
      MOV       CX, AnchFig               ; Nmero de columnas de la figura
      MOV       AL, 0
      REPZ      SCASB                     ; Repetimos mientras encontremos ceros
      JNZ       $_FinMal                  ; Si encontramos un byte no cero => ZF=0
      DEC       DX                        ; Decrementamos 1 en las alturas a chequear
    JNS       $_BucleF
    XOR        AL, AL
    STC                                   ; Todo bien
    JMP       $_Fin                       ; Si hemos llegado aqu, todo ha ido bien
    $_FinMal:
    CLC                                   ; Ha ido mal
    $_Fin:
    POP       ES
    POP       DS
    POP       BX
    RET
  CheckInf ENDP

  CheckLeft PROC
    ; Propsito: Chequeamos si la figura sobrepas la parte izquierda del tablero
    ; entrada  : BX: Apunta a la definicin real de la figura
    ; salida   : ZeroFlag = 1 -> est bien, ZeroFlag = 0 -> est mal
    ; Destruye : Ninguna
    PUSH      BX
    PUSH      DS
    PUSH      ES
    MOV       AX, CS
    MOV       DS, AX
    MOV       ES, AX
    MOV       DX, WORD PTR [miFigura.PosXtmp]
    NEG       DX
    JLE       $_FinOK
    $_BucleF:
      MOV       CX, AltFig                ; Nmero de filas de la figura
      MOV       SI, BX                    ; Recogemos la columna de la figura
      $_BucleC:
        OR        BYTE PTR DS:[SI], 0
        JNZ       $_Fin                   ; Si encontramos byte no cero, ZF=0, salimos
        ADD       SI, AnchFig             ; Siguiente fila
        DEC       CX                      ; Decrementamos 1 en la fila
      JNZ       $_BucleC
      INC       BX                        ; Siguiente columna en figura
      DEC       DX                        ; Decrementamos 1 en las columnas a chequear
    JNZ       $_BucleF
    $_FinOK:                              
    XOR       AL, AL
    STC                                   ; Todo bien
    $_Fin:
    POP       ES
    POP       DS
    POP       BX
    RET
  CheckLeft ENDP

  CheckRight PROC
    ; Propsito: Chequeamos si la figura sobrepas la parte derecha del tablero
    ; entrada  : BX: Apunta a la definicin real de la figura
    ; salida   : ZeroFlag = 1 -> est bien, ZeroFlag = 0 -> est mal
    ; Destruye : Ninguna
    PUSH      BX
    PUSH      DS
    PUSH      ES
    MOV       AX, CS                      ; Necesario por la ISR del reloj
    MOV       DS, AX
    MOV       ES, AX
    MOV       DX, miFigura.PosXtmp
    ADD       DX, AnchFig - Columnas      ; DX = AnchFig - (Columnas - miFigura.PosX)
    JLE       $_FinOK
    ADD       BX, AnchFig - 1             ; Apuntamos a la ltima columna de la figura
    $_BucleF:
      MOV       CX, AltFig                ; Nmero de filas de la figura
      MOV       SI, BX                    ; Recogemos la columna de la figura
      $_BucleC:
        OR        BYTE PTR DS:[SI], 0
        JNZ       $_Fin                   ; Si encontramos byte no cero, ZF=0, salimos
        ADD       SI, AnchFig             ; Siguiente fila
        DEC       CX                      ; Decrementamos 1 en la fila
      JNZ       $_BucleC
      DEC       BX                        ; Anterior columna en figura
      DEC       DX                        ; Decrementamos 1 en las columnas a chequear
    JNZ       $_BucleF
    $_FinOK:
    XOR        AL, AL
    STC                                   ; Todo bien
    $_Fin:
    POP       ES
    POP       DS
    POP       BX
    RET
  CheckRight ENDP
  
  EsPosibleMvto PROC
    ; Propsito: Chequeamos el movimiento para ver si es posible
    ; entrada  : Ninguna
    ; salida   : ZeroFlag = 1 -> est bien, ZeroFlag = 0 -> est mal
    ;            CarryFlag = 1 -> con vida, CarryFlag = 0 -> fin del movimiento
    ; Destruye : Ninguna
    PUSH      AX
    PUSH      BX
    PUSH      CX
    PUSH      DX
    PUSH      SI
    PUSH      DI
    PUSH      DS
    PUSH      ES
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    ; 0) Parte troncal. BX: Apunta a la definicin real de la figura
    MOV       BX, WORD PTR [miFigura.Tipo]
    SHL       BX, 1                       ; Porque tratamos con words
    MOV       SI, OFFSET PosFigs          ; Direccin de matriz de direcciones
    MOV       SI, WORD PTR [SI+BX]        ; SI apunta a la 1 direccin en L1,L2,Z1,Z2,B1,C1
    MOV       BX, WORD PTR [miFigura.Rotatmp]
    SHL       BX, 1                       ; Porque tratamos con words
    MOV       BX, WORD PTR [SI+BX]        ; BX apunta a la BX-definicin de la figura
    ; 1) Comprobamos los lmites
    ; 1.1) Parte inferior
    CALL      CheckInf                    ; Chequeamos si sobrepas la parte inferior
    JNC       $_Fin_CF                    ; Si CF=0, se sobrepas (mal)
    ; 1.2) Izquierda
    CALL      CheckLeft                   ; Chequeamos si sobrepas la parte izquierda
    STC
    JNZ       $_Fin                       ; Si ZF=0, se sobrepas, hemos terminado
    ; 1.3) Derecha
    CALL      CheckRight                  ; Chequeamos si sobrepas la parte derecha
    STC
    JNZ       $_Fin                       ; Si ZF=0, se sobrepas, hemos terminado
    ; 2) Comprobamos si la figura pisa algo sobre el tablero => movimiento incorrecto
    CALL      Superpone
    STC                                   ; Si llegamos hasta aqu debemos suponer STC
    JZ        $_Fin                       ; Si ZF=1, todo correcto
    $_Fin_CF:
      ; Comprobamos si el motivo de la superposicin es porque lleg al final
      ; (si el movimiento era de bajada y hubo superposicin)
      OR        AL, 1                     ; Suponemos que no hemos llegado al final, sino mal mvto
      CMP       BYTE PTR [Movimiento], 1  ; Si habamos pedido bajar
      STC
      JNE       $_Fin
        CLC                               ;   es que ha llegado al final
        JMP     $_Fin
    $_Fin_Ok:
      XOR       AL, AL
      STC
    $_Fin:
    POP       ES
    POP       DS
    POP       DI
    POP       SI
    POP       DX
    POP       CX
    POP       BX
    POP       AX
    RET
  EsPosibleMvto ENDP

  MueveFicha PROC
    ; Propsito: Mueve la ficha dentro de Tablero y lo vuelca a la pantalla
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    CALL      CopiaTableroATmp    ; Copia Tablero a TableroTmp
    CALL      ColocaFigura        ; Movemos la figura en Tablero
    CALL      TableroTmp2Scrn        ; Volcamos el contenido de Tablero en la pantalla
    MOV       BYTE PTR [Movimiento], 0    ; Reinicializamos la bandera
    RET
  MueveFicha ENDP

  CopiaTmpATablero PROC
    ; Propsito: Vuelca TableroTmp a Tablero
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       CX, Filas*Columnas/2        ; Tenemos que haberlo definido par, sino...
    MOV       SI, OFFSET TableroTmp
    MOV       DI, OFFSET Tablero
    CLD
    REP       MOVSW
    POP       DS
    POP       ES
    RET
  CopiaTmpATablero ENDP

  CopiaTableroATmp PROC
    ; Propsito: Vuelca Tablero a TableroTmp
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       CX, Filas*Columnas/2        ; Tenemos que haberlo definido par, sino...
    CLD
    MOV       SI, OFFSET Tablero
    MOV       DI, OFFSET TableroTmp
    REP       MOVSW
    POP       DS
    POP       ES
    RET
  CopiaTableroATmp ENDP

  NuevasCC PROC
    ; Propsito: Establecemos las nuevas coordenadas despus de haber visto que son correctas
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       AX, WORD PTR [miFigura.PosXtmp]
    MOV       WORD PTR [miFigura.PosX], AX
    MOV       AX, WORD PTR [miFigura.PosYtmp]
    MOV       WORD PTR [miFigura.PosY], AX
    MOV       AX, WORD PTR [miFigura.Rotatmp]
    MOV       WORD PTR [miFigura.Rota], AX
    POP       DS
    POP       ES
    POP       AX
    RET
  NuevasCC ENDP
  
  GuardaCC PROC
    ; Propsito: Guarda las coordenadas actuales en tmp
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      AX
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    MOV       AX, WORD PTR [miFigura.PosX]
    MOV       WORD PTR [miFigura.PosXtmp], AX
    MOV       AX, WORD PTR [miFigura.PosY]
    MOV       WORD PTR [miFigura.PosYtmp], AX
    MOV       AX, WORD PTR [miFigura.Rota]
    MOV       WORD PTR [miFigura.Rotatmp], AX
    POP       DS
    POP       ES
    POP       AX
    RET
  GuardaCC ENDP

  GetKey PROC
    ; Propsito: Recoger las pulsaciones de las teclas y modificar "Movimiento"
    ; entrada  : Ninguna
    ; salida   : Ninguna
    ; Destruye : Ninguna
    PUSH      ES
    PUSH      DS
    MOV       AX, CS
    MOV       ES, AX
    MOV       DS, AX
    OR        BYTE PTR [Movimiento], 0
    JNZ       $_Fin
    MOV       AH, 1                     ; Recogemos tecla, pero no esperamos
    INT       16h
    JNZ       $_Distingue_Tecla
      JMP       $_Fin
    $_Distingue_Tecla:
    XOR       AH, AH
    INT       16h
    CALL      GuardaCC                  ; Volcamos la situacin actual
    $_Tecla_Abajo:
    CMP       AH, 80
    JNE       $_Tecla_Izquierda
      INC       WORD PTR [miFigura.PosYtmp]
      MOV       BYTE PTR [Movimiento], 1  ; Hemos solicitado hacia abajo
      JMP       $_Fin
    $_Tecla_Izquierda:
    CMP       AH, 75
    JNE       $_Tecla_Arriba
      DEC       WORD PTR [miFigura.PosXtmp]
      MOV       BYTE PTR [Movimiento], 2  ; Hemos solicitado hacia izquierda
      JMP       $_Fin
    $_Tecla_Arriba:
    CMP       AH, 77
    JNE       $_Tecla_Derecha
      INC       WORD PTR [miFigura.PosXtmp]
      MOV       BYTE PTR [Movimiento], 4  ; Hemos solicitado hacia derecha
      JMP       $_Fin
    $_Tecla_Derecha:
    CMP       AH, 72
    JNE       $_Tecla_Nueva_Partida
      INC       WORD PTR [miFigura.Rotatmp]
      .IF       WORD PTR [miFigura.Rotatmp] >= NumRota
        MOV       WORD PTR [miFigura.Rotatmp], 0
      .ENDIF
      MOV       BYTE PTR [Movimiento], 3  ; Hemos solicitado rotar
      JMP       $_Fin
    $_Tecla_Nueva_Partida:
    OR        AL, 0100000b                 ; Lo convertimos a minsculas
    CMP       AL, 'n'
    JNE       $_Tecla_Fin_Juego
      MOV       BYTE PTR [Movimiento], -1  ; Hemos solicitado nueva partida
      JMP       $_Fin
    $_Tecla_Fin_Juego:
    CMP       AH, 1    ; ESC - Exit
    JNZ       $_Fin
      MOV       BYTE PTR [Movimiento], -2  ; Hemos solicitado fin del juego
    $_Fin:
    POP       DS
    POP       ES
    RET
  GetKey ENDP

  ; -------------------------------------------------------
  ; - Datos                                               -
  ; -------------------------------------------------------
  ; ************ figuras *************
  L11   DB 0,1,1,1        ; XXX;
        DB 0,1,0,0        ; X  ;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L12   DB 0,1,1,0        ; XX ;
        DB 0,0,1,0        ;  X ;
        DB 0,0,1,0        ;  X ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L13   DB 0,0,0,1        ;   X;
        DB 0,1,1,1        ; XXX;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L14   DB 0,1,0,0        ; X  ;
        DB 0,1,0,0        ; X  ;
        DB 0,1,1,0        ; XX ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  L21   DB 0,1,0,0        ; X  ;
        DB 0,1,1,1        ; XXX;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L22   DB 0,1,1,0        ; XX ;
        DB 0,1,0,0        ; X  ;
        DB 0,1,0,0        ; X  ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L23   DB 0,1,1,1        ; XXX;
        DB 0,0,0,1        ;   X;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  L24   DB 0,0,1,0        ;  X ;
        DB 0,0,1,0        ;  X ;
        DB 0,1,1,0        ; XX ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  E11   DB 0,1,1,1        ; XXX;
        DB 0,0,1,0        ;  X ;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  E12   DB 0,0,0,1        ;   X;
        DB 0,0,1,1        ;  XX;
        DB 0,0,0,1        ;   X;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  E13   DB 0,0,0,0        ;    ;
        DB 0,0,1,0        ;  X ;
        DB 0,1,1,1        ; XXX;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  E14   DB 0,1,0,0        ; X  ;
        DB 0,1,1,0        ; XX ;
        DB 0,1,0,0        ; X  ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  U11   DB 0,1,1,1        ; XXX;
        DB 0,1,0,1        ; X X;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  U12   DB 0,0,1,1        ;  XX;
        DB 0,0,0,1        ;   X;
        DB 0,0,1,1        ;  XX;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  U13   DB 0,0,0,0        ;    ;
        DB 0,1,0,1        ; X X;
        DB 0,1,1,1        ; XXX;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  U14   DB 0,1,1,0        ; XX ;
        DB 0,1,0,0        ; X  ;
        DB 0,1,1,0        ; XX ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  Z11   DB 0,1,0,0        ; X  ;
        DB 0,1,1,0        ; XX ;
        DB 0,0,1,0        ;  X ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  Z12   DB 0,0,1,1        ;  XX;
        DB 0,1,1,0        ; XX ;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  Z21   DB 0,0,1,0        ;  X ;
        DB 0,1,1,0        ; XX ;
        DB 0,1,0,0        ; X  ;
        DB 0,0,0,0        ;    ;
  ; --------------------------------
  Z22   DB 0,1,1,0        ; XX ;
        DB 0,0,1,1        ;  XX;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  B11   DB 0,0,1,0        ;  X ;
        DB 0,0,1,0        ;  X ;
        DB 0,0,1,0        ;  X ;
        DB 0,0,1,0        ;  X ;
  ; --------------------------------
  B12   DB 1,1,1,1        ;XXXX;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
        DB 0,0,0,0        ;    ;
  ; ===================================
  C11   DB 0,0,0,0        ;    ;
        DB 0,1,1,0        ; XX ;
        DB 0,1,1,0        ; XX ;
        DB 0,0,0,0        ;    ;
  ; ===================================
;   O11   DB 0,1,1,1        ; XXX;    ; Con esta es supercomplicado, la quitamos
;         DB 0,1,0,1        ; X X;
;         DB 0,1,1,1        ; XXX;
;         DB 0,0,0,0        ;    ;
  ; ===================================
  ; Direcciones de las figuras
  L1    DW OFFSET L11, OFFSET L12, OFFSET L13, OFFSET L14    ; L1
  L2    DW OFFSET L21, OFFSET L22, OFFSET L23, OFFSET L24    ; L2
  E1    DW OFFSET E11, OFFSET E12, OFFSET E13, OFFSET E14    ; E1
  U1    DW OFFSET U11, OFFSET U12, OFFSET U13, OFFSET U14    ; U1
  Z1    DW OFFSET Z11, OFFSET Z12, OFFSET Z11, OFFSET Z12    ; Z1
  Z2    DW OFFSET Z21, OFFSET Z22, OFFSET Z21, OFFSET Z22    ; Z2
  B1    DW OFFSET B11, OFFSET B12, OFFSET B11, OFFSET B12    ; B1
  C1    DW OFFSET C11, OFFSET C11, OFFSET C11, OFFSET C11    ; C1
;   O1    DW OFFSET O11, OFFSET O11, OFFSET O11, OFFSET O11    ; O1
  PosFigs     DW OFFSET L1, OFFSET L2, OFFSET E1, OFFSET U1
              DW OFFSET Z1, OFFSET Z2, OFFSET B1, OFFSET C1;, OFFSET O1
  Figura      DB ?, ?                     ; Figura que pintamos y la siguiente
;   align       16
;   db 0,0,0,0,0,0,0,0
;   db Columnas dup(0FFh)  ; Con esto enmarcamos el Tablero *** slo para depurar ***
  Tablero     DB Filas*Columnas   DUP(?)  ; Contiene la deficinin del tablero definitivo
;   db Columnas dup(0FFh)  ; Con esto enmarcamos el Tablero *** slo para depurar ***
  TableroTmp  DB Filas*Columnas   DUP(?)  ; Aqu entra la figura actual
  TableroBuf  DB Filas*Columnas*2 DUP(?)  ; Modo raw para despus volcar a scrn
  miFigura    FigReg <>                   ; miFigura sin inicializar
  OldInt1Ch   DD  ?                       ; Direccin antigua de Int 1Ch
  Velocidad   DW  ?                       ; Ir disminuyendo segn aumente el nivel
  Tiempo      DW  ?                       ; Contador de pulsos que rellenar Velocidad
  Movimiento  DB  ?                       ; Movimiento de la figura 1:abajo, 2:izqda, 3:rotar, 4:derecha
                                          ; -1:nueva partida, -2:salir, -3:fin movimiento ficha
  Texto1      DB   "TETRIS", 0
  Texto2      DB   "    Alfonso Victor Caballero Hurtado$"
  Texto3      DB   "    (c) Abre los ojos al ensamblador$"
  Texto4      DB   "            100% assembly code$"
  Texto5      DB   "Do you want to learn assembly programming?$"
  Texto6      DB   "    http://www.abreojosensamblador.net$"
  Texto7      DB   "Next:$"
  Texto8      DB   "Level:$"
  Texto9      DB   "Score:$"
  DEFSEG      DW   ?
  DEFOFF      DW   ?
  Nivel       DB   ?
  Marcador    DW   ?
  ContLineas  DB   ?                      ; Contador de lneas por nivel
  TamCursor   DW   ?
  Blanco      DB   __LongMax DUP(" "), "$"
codigo ENDS
END Entrada
 
